#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

# Values come from the Redfish schema:
# https://redfish.dmtf.org/schemas/v1/MessageRegistry.v1_4_2.json

import enum


class MessageParamType(enum.Enum):
    """Message Registry message parameter type related constants."""
    STRING = "string"
    NUMBER = "number"
